/*
    This file may be used for any purpose whatsoever, provided that the
    original author is never misrepresented.
  
    Coded by Bastiaan Veelo in 2004
*/

module widgets;

private import dcouple.signalslot;

//class Widget : SignalManager, SlotManager
class Widget : SignalSlotManager
{
	this(Widget parent)
	{
		_parent = parent;
		if(parent) parent._children[this] = this;
	}

	/*! SignalManager implementation */
	void register(GenericSignal s)
	{
		_signals[s] = s;
	}
	
	/*! SlotManager implementation */
	void register(GenericSlot s)
	{
		_slots[s] = s;
	}
		
	void discard()
	{
		foreach(Widget w; _children) w.discard();
		foreach(GenericSignal s; _signals) s.disconnect();
		foreach(GenericSlot s; _slots) s.disconnect();
	}
	
private:
	Widget _parent;
	Widget[Widget] _children;
	GenericSignal[GenericSignal] _signals;
	GenericSlot[GenericSlot] _slots;
}

class Button : Widget
{
	Signal!() pressed;
	
	void press()
	{
		pressed.emit();
	}
	
	this(Widget parent)
	{
		super(parent);
		pressed = new Signal!()(this);
		debug printf("Button constructed.\n");
	}
	~this()
	{
		debug printf("Button deleted.\n");
	}
}

class ResetButton : Widget
{
	Signal!(int) pressed;
	
	void press()
	{
		pressed.emit(0);
	}
	
	this(Widget parent)
	{
		super(parent);
		pressed = new Signal!(int)(this);
	}
	~this()
	{
		debug printf("resetButton deleted.\n");
	}
}

class SpinBox : Widget
{
	void increase()
	{
		val++;
		printf( "%d\n", val);
	}
	
	void decrease()
	{
		val--;
		printf( "%d\n", val);
	}
	
	void set(int i ) 
	{ 
		val = i;
		printf( "%d\n", val);
	}
	
	Slot!() up;
	Slot!() down;
	Slot!(int) setVal;
	
	this(Widget parent)
	{
		printf("Constructing SpinBox.\n");
		super(parent);
		up = new Slot!()(this, &increase);
		down = new Slot!()(this, &decrease);
		setVal = new Slot!(int)(this, &set);
	}
	~this()
	{
		debug printf("SpinBox deleted.\n");
	}
private:
	int val;
}

class Foo { }

class NastyWidget : Widget
{
	Signal!(Foo,char[]) sig;
	this(Widget parent)
	{
		super( parent );
		sig = new Signal!(Foo,char[])(this);
	}
	~this()
	{
		debug printf("NastyWidget deleted.\n");
	}
}
