import widgets;
import dcouple.connect;

import std.gc;

void main()
{
	// Some widgets. They are all top-level here, so no parents defined.
	// SpinBox has "up" and "down" slots.
	SpinBox moneyInTheBank = new SpinBox(null);
	// Button has "pressed" signal.
	Button incomeButton = new Button(null);		// Button has a signal.
	Button expenseButton = new Button(null);
	
	// This syntax is possible after import dcouple.connect.
	connect(incomeButton.pressed, moneyInTheBank.up);
	
	// Alternatively, these are also equivalent:
	incomeButton.pressed.connect(moneyInTheBank.up);
	moneyInTheBank.up.connect(incomeButton.pressed);
	
	// Multiple identical connections have no effect.
	
	// Make one more connection.
	connect(expenseButton.pressed, moneyInTheBank.down);
	
	// Some action please. Button.press() is an ordinary member function,
	// calling Signal.emit().
	incomeButton.press();
	expenseButton.press();
	
	// Explicitly disconnecting:
	disconnect(incomeButton.pressed, moneyInTheBank.up);
	// or:
	incomeButton.pressed.disconnect(moneyInTheBank.up);
	// or:
	moneyInTheBank.up.disconnect(incomeButton.pressed);
	// Disconnecting non-existing connections has no effect.
	
	incomeButton.press(); // does nothing now.

	// Demo for the doubters:
	printf("re-connecting.\n");
	incomeButton.pressed.connect(moneyInTheBank.up);
	incomeButton.press();
	
	disconnect(incomeButton.pressed, moneyInTheBank.up);
	
	printf("re-connecting.\n");
	moneyInTheBank.up.connect(incomeButton.pressed);
	incomeButton.press();
	
	// Testing GC. This may not give the expected result, because GC code has 
	// no deterministic finalisation.
	moneyInTheBank.discard();
	moneyInTheBank = null;
	incomeButton.press();	// Allowed.
	fullCollect();
	incomeButton.press();	// Allowed.
	
	printf("THE END.\n");
}