module widgets;

private import dcouple.signalslot;

class Widget : SignalManager, SlotManager
{
	this(Widget parent)
	{
		_parent = parent;
		if(parent) parent._children[this] = this;
	}

	~this()
	{
		discard();
	}
	
	/*! SignalManager implementation */
	void register(Signal s)
	{
		_signals[s] = s;
	}
	
	void deregister(Signal s)
	{
		delete _signals[s];
	}

	/*! SlotManager implementation */
	void register(Slot s)
	{
		_slots[s] = s;
	}
	
	void deregister(Slot s)
	{
		delete _slots[s];
	}
	
	void discard()
	{
		debug(3) printf("in Widget.discard().\n");
		foreach(Widget w; _children) w.discard();
		debug(3) printf("children discard()ed.\n");
		debug(3) printf("%d signal(s) registered.\n", _signals.length);
		foreach(Signal s; _signals) s.disconnect();
		debug(3) printf("signals disconnect()ed.\n");
		foreach(Slot s; _slots) s.disconnect();
		debug(3) printf("slots disconnect()ed.\n");
	}
	
private:
	Widget _parent;
	Widget[Widget] _children;
	Signal[Signal] _signals;
	Slot[Slot] _slots;
}

class Button : Widget
{
	Signal pressed;
	
	void press()
	{
		pressed.emit();
	}
	
	this(Widget parent)
	{
		super(parent);
		pressed = new Signal(this);
		debug(2) printf("Button constructed.\n");
	}
	~this()
	{
		debug(1) printf("Button deleted.\n");
	}
}

class SpinBox : Widget
{
	void increase()
	{
		val++;
		printf( "%d\n", val);
	}
	
	void decrease()
	{
		val--;
		printf( "%d\n", val);
	}
	
	Slot up;
	Slot down;
	
	this(Widget parent)
	{
		super(parent);
		up = new Slot(this, &increase);
		down = new Slot(this, &decrease);
		debug(2) printf("SpinBox constructed.\n");
	}
	~this()
	{
		debug(1) printf("SpinBox deleted.\n");
	}
private:
	int val;
}
